####################################################################################################
# This R-script performs individual baed simulations to estimate the probability of a new          #
# mutation in a changing environment.                                                              #
#                                                                                                  #
# Fitness changes in generation n is given by                                                      #
#                                                                                                  #
#  1 + s + ds * cos(n)*(alpha * n + phi)                                                           #   
#                                                                                                  #
#                                                                                                  #  
# The script plots the probability of establishment as a function of the parameter alpha.          #
# The plots show the probability of establishment calcualted from the simulations, the             #
# correpsonding 99% confidence intervals, and the analytical prediciton as given in Peischl        #
# and Kirkpatrick (2012).                                                                          #
#                                                                                                  #
####################################################################################################


#population size 
pop_size<-50000

#growth rate 
r<-2

#maximum number of generations 
generations<-5000

#number of fixation events
number_fix<-0

#number of replicates per parameter combination 
reps<-10000

#selection paramters
s<-0.01
ds <- 0.005
phi<-0

#number of parameter values for s0
steps <- 5


alphavec <-mat.or.vec(1,steps)
fix_vec <- mat.or.vec(1,steps)

#values of alpha 
alphavec <- c(0.0001,0.001,0.01,0.1,1)

for (jj in c(1:steps))   # loop that goes through all values of alphavec
{
  alpha <- alphavec[jj]   # pick alpha 
  number_fix<-0

  for (ii in c(1:reps))   # simulate replicates for one value of alpha
  { 

 
    muts <- 1                   # initial number of mutations = 1
    resi <- pop_size - muts     # initial number of residents
    tot_ind<-pop_size           # total number of individuals  

    
    for(g in c(0:generations))    
    {
      # calculate selection coefficient of generation g
      sn <- s + ds * cos(alpha*g + phi)

      
      # calculate expected numer of offspring of a resident 
      lambda <- r/( 1 + (tot_ind*(r-1)/pop_size))
    
      # calculate expected number of offspring of a mutant
      lambda_mut <-lambda*(1+sn)
    
      # mutant offspring
      muts <- sum(rpois(muts,lambda_mut))

      # resident offspring
      resi <- sum(rpois(resi,lambda))
     
      tot_ind<- muts + resi
      
      if (muts > 500)   # we stop if there are more than 500 mutants alive and count this as fixation 
      {
        number_fix <- number_fix+1
        break
      }
      
      
      if (muts==0)   # if no mutant leaves descendants we stop and count this as extinction
      {
    
        break
  
      }
    
    }
  
  }
  fix_vec[jj]<-number_fix/reps  # calculate fixation probability 
}

qv<-fix_vec   
w<-2.57* sqrt(qv *(1-qv)/reps) # calcualte 99% confidence interval

#plot confidence intervals
plotCI(alphavec,qv,uiw=w,col="black",ylim=c(0.0,0.035),xlab=expression(alpha),ylab=expression(p),lwd=2,pch=20,gap=0,log="x")

#plot analytical prediction of effective seleciton coefficient
alphas <-  10^((c(1:(10*(steps))-10)/10))*alphavec[1]
lines(alphas,2*s*(1+ds*((1-s)*cos(alphas-phi)-cos(phi))/(2*(1-s)*cos(alphas)-((1-s)^2)-1)),lwd=2,lty=1,log="x")